package org.gcube.portlets.user.dataminermanager.client.rpc;

import java.util.List;
import java.util.Map;

import org.gcube.portlets.user.dataminermanager.client.bean.ComputationStatus;
import org.gcube.portlets.user.dataminermanager.client.bean.Operator;
import org.gcube.portlets.user.dataminermanager.client.bean.OperatorsClassification;
import org.gcube.portlets.user.dataminermanager.client.bean.output.ImagesResource;
import org.gcube.portlets.user.dataminermanager.client.bean.output.MapResource;
import org.gcube.portlets.user.dataminermanager.client.bean.output.Resource;
import org.gcube.portlets.user.dataminermanager.client.bean.parameters.Parameter;
import org.gcube.portlets.user.dataminermanager.shared.data.ComputationId;
import org.gcube.portlets.user.dataminermanager.shared.data.TableItemSimple;
import org.gcube.portlets.user.dataminermanager.shared.exception.ServiceException;
import org.gcube.portlets.user.dataminermanager.shared.session.UserInfo;
import org.gcube.portlets.user.dataminermanager.shared.workspace.DataMinerWorkArea;
import org.gcube.portlets.user.dataminermanager.shared.workspace.ItemDescription;
import org.gcube.portlets.widgets.wsexplorer.shared.Item;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 */
@RemoteServiceRelativePath("dataminerman")
public interface DataMinerPortletService extends RemoteService {

	/*
	// get tables info belongs to a given templates set (for relative user)
	public List<TableItemSimple> getTableItems(List<String> templates)
			throws Exception;
	*/
	public List<TableItemSimple> getFileItems(List<String> templates)
			throws Exception;

	public Resource getResourceByComputationId(ComputationId computationId);

	public Map<String, String> getParametersMapByComputationId(ComputationId computationId) throws ServiceException;

	
	public Map<String, Resource> getMapFromMapResource(MapResource mapResource)
			throws Exception;

	// Map<String, String> getImagesInfoFromImagesResource(ImagesResource
	// imgsRes) throws Exception;

	public String saveImage(ComputationId computationId, ImagesResource imageResource)
			throws Exception;

	public void removeResource(String id) throws Exception;

	public void removeImport(String id) throws Exception;
	
	
	/**
	 * Dataminer
	 * 
	 */
	public UserInfo hello() throws ServiceException;

	public List<OperatorsClassification> getOperatorsClassifications()
			throws Exception;

	public List<Parameter> getParameters(Operator operator) throws Exception;

	public ComputationId startComputation(Operator op, String computationTitle,
			String computationDescription) throws Exception;

	public ComputationStatus getComputationStatus(ComputationId computationId)
			throws Exception;

	
	public ComputationId resubmit(ComputationId computationId) throws ServiceException;

	
	public TableItemSimple retrieveTableInformation(Item item)
			throws ServiceException;

	public DataMinerWorkArea getDataMinerWorkArea() throws ServiceException;

	public String getPublicLink(ItemDescription itemDescription)
			throws ServiceException;

	public String cancelComputation(ComputationId computationId) throws ServiceException;

}
